# Generates a Makefile that:
# 1. Cleans up the build directory
# 2. Runs cmake to build qtbind
# 3. Copies the built .so files into the correct places

windows = false
macosx  = false
processor, platform, *rest = RUBY_PLATFORM.split("-")
windows = true if platform == 'mswin32' or platform == 'mingw32'
macosx  = true if platform =~ /darwin/

ruby_version = '1.9'
ruby_version = '1.8' if RUBY_VERSION.split('.')[1].to_i == 8

def find_qt_sdk
  begin
    entries = Dir.entries("C:\\Qt")
  rescue
    raise "Could not find Qt SDK in C:\\Qt"
  end
  entries.sort!
  "C:\\Qt\\" + entries[-1]
end

qt_sdk_path = find_qt_sdk() if windows

File.open('Makefile', 'w') do |file|
  if windows
    file.puts "all: clean build"
    file.puts ""
    file.puts "makedirs:"
    file.puts "\t-mkdir ext\\build"
    file.puts "\t-mkdir bin\\1.8"
    file.puts "\t-mkdir bin\\1.9"
    file.puts "\t-mkdir bin\\plugins"
    file.puts "\t-mkdir bin\\plugins\\accessible"
    file.puts "\t-mkdir bin\\plugins\\codecs"
    file.puts "\t-mkdir bin\\plugins\\designer"
    file.puts "\t-mkdir bin\\plugins\\graphicssystems"
    file.puts "\t-mkdir bin\\plugins\\iconengines"
    file.puts "\t-mkdir bin\\plugins\\imageformats"
    file.puts "\t-mkdir bin\\plugins\\phonon_backend"
    file.puts "\t-mkdir bin\\plugins\\sqldrivers"
    file.puts "\t-mkdir lib\\1.8"
    file.puts "\t-mkdir lib\\1.9"
    file.puts ""    
    file.puts "clean: makedirs"
    file.puts "\t-cd ext\\build && rmdir /S /Q CMakeFiles"
    file.puts "\t-cd ext\\build && rmdir /S /Q generator"
    file.puts "\t-cd ext\\build && rmdir /S /Q smoke"
    file.puts "\t-cd ext\\build && rmdir /S /Q ruby"
    file.puts "\t-cd ext\\build && del /F /Q *"
    file.puts ""
    file.puts "distclean: clean" 
    file.puts "\t-cd bin && del /F /Q *.dll"
    file.puts "\t-cd bin && del /F /Q *.so"
    file.puts "\t-cd bin && del /F /Q *.exe"
    file.puts "\t-cd bin\\plugins && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\accessible && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\codecs && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\designer && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\graphicssystems && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\iconengines && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\imageformats && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\phonon_backend && del /F /Q *"
    file.puts "\t-cd bin\\plugins\\sqldrivers && del /F /Q *"
    file.puts "\t-cd bin\\1.8 && del /F /Q *"
    file.puts "\t-cd bin\\1.9 && del /F /Q *"
    file.puts "\t-cd lib\\1.8 && del /F /Q *"
    file.puts "\t-cd lib\\1.9 && del /F /Q *"
    file.puts "\t-del /F /Q Makefile"
    file.puts "\t-del /F /Q qtbindings-*.gem"
    file.puts ""
    file.puts "build: makedirs"
    file.puts "\tset CC=mingw32-gcc.exe"
    file.puts "\tset CXX=mingw32-g++.exe"  
    file.puts "\t-cd ext\\build && \\"
    file.puts "cmake \\"
    file.puts "-G \"MinGW Makefiles\" \\"
    if ARGV[0] == '-d'
      file.puts "-DCMAKE_BUILD_TYPE=Debug \\"
    end
    file.puts "-DCMAKE_MAKE_PROGRAM=mingw32-make.exe \\"
    file.puts "-Wno-dev \\"
    file.puts "-DENABLE_SMOKE=on \\"
    file.puts "-DENABLE_QTCORE_SMOKE=on \\"
    file.puts "-DENABLE_QTNETWORK_SMOKE=on \\"
    file.puts "-DENABLE_QTDBUS_SMOKE=off \\"
    file.puts "-DENABLE_QTGUI_SMOKE=on \\"
    file.puts "-DENABLE_QTSVG_SMOKE=on \\"
    file.puts "-DENABLE_QTSQL_SMOKE=on \\"
    file.puts "-DENABLE_QTXML_SMOKE=on \\"
    file.puts "-DENABLE_QTXMLPATTERNS_SMOKE=on \\"
    file.puts "-DENABLE_QTOPENGL_SMOKE=on \\"
    file.puts "-DENABLE_QTWEBKIT_SMOKE=on \\"
    file.puts "-DENABLE_QTSCRIPT_SMOKE=on \\"
    file.puts "-DENABLE_QTUITOOLS_SMOKE=on \\"
    file.puts "-DENABLE_QTTEST_SMOKE=on \\"
    file.puts "-DENABLE_QTMULTIMEDIA_SMOKE=on \\"
    file.puts "-DENABLE_QTRUBY=on \\"
    file.puts "-DENABLE_QTWEBKIT_RUBY=on \\"
    file.puts "-DENABLE_QTUITOOLS_RUBY=on \\"
    file.puts "-DENABLE_QTSCRIPT=on \\"
    file.puts "-DENABLE_QTTEST=on \\"
    file.puts ".."
    file.puts "\tcd ext\\build && mingw32-make"
    file.puts ""
    file.puts "install: makedirs"
    file.puts "\t-copy ext\\build\\smoke\\deptool\\smokedeptool.exe bin\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtcore\\libsmokeqtcore.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtgui\\libsmokeqtgui.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qthelp\\libsmokeqthelp.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtmultimedia\\libsmokeqtmultimedia.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtnetwork\\libsmokeqtnetwork.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtopengl\\libsmokeqtopengl.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtscript\\libsmokeqtscript.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtsql\\libsmokeqtsql.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtsvg\\libsmokeqtsvg.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qttest\\libsmokeqttest.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtuitools\\libsmokeqtuitools.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtwebkit\\libsmokeqtwebkit.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtxml\\libsmokeqtxml.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\qtxmlpatterns\\libsmokeqtxmlpatterns.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\smokeapi\\smokeapi.exe bin\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\smoke\\smokebase\\libsmokebase.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\ruby\\qtruby\\src\\libqtruby4shared.dll lib\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\ruby\\qtruby\\src\\qtruby4.dll lib\\#{ruby_version}\\qtruby4.so"
    file.puts "\t-copy ext\\build\\ruby\\qtscript\\qtscript.dll lib\\#{ruby_version}\\qtscript.so"
    file.puts "\t-copy ext\\build\\ruby\\qttest\\qttest.dll lib\\#{ruby_version}\\qttest.so"
    file.puts "\t-copy ext\\build\\ruby\\qtuitools\\qtuitools.dll lib\\#{ruby_version}\\qtuitools.so"
    file.puts "\t-copy ext\\build\\ruby\\qtwebkit\\qtwebkit.dll lib\\#{ruby_version}\\qtwebkit.so"
    file.puts "\t-copy ext\\build\\ruby\\qtruby\\tools\\rbrcc\\rbrcc.exe bin\\#{ruby_version}"
    file.puts "\t-copy ext\\build\\ruby\\qtruby\\tools\\rbuic\\rbuic4.exe bin\\#{ruby_version}"
    file.puts ""
    file.puts "installqt: makedirs"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\bin\\*.dll bin"
    file.puts "\tdel /F /Q bin\\*d4.dll"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\accessible\\*.dll bin\\plugins\\accessible"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\codecs\\*.dll bin\\plugins\\codecs"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\designer\\*.dll bin\\plugins\\designer"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\graphicssystems\\*.dll bin\\plugins\\graphicssystems"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\iconengines\\*.dll bin\\plugins\\iconengines"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\imageformats\\*.dll bin\\plugins\\imageformats"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\phonon_backend\\*.dll bin\\plugins\\phonon_backend"
    file.puts "\tcopy #{qt_sdk_path}\\qt\\plugins\\sqldrivers\\*.dll bin\\plugins\\sqldrivers"
    file.puts "\tdel /F /Q bin\\plugins\\accessible\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\accessible\\*d4.dll"
    file.puts "\tdel /F /Q bin\\plugins\\codecs\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\codecs\\*d4.dll"
    file.puts "\tdel /F /Q bin\\plugins\\designer\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\designer\\*d4.dll"
    file.puts "\tdel /F /Q bin\\plugins\\graphicssystems\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\graphicssystems\\*d4.dll"
    file.puts "\tdel /F /Q bin\\plugins\\iconengines\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\iconengines\\*d4.dll"
    file.puts "\tdel /F /Q bin\\plugins\\imageformats\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\imageformats\\*d4.dll"
    file.puts "\tdel /F /Q bin\\plugins\\phonon_backend\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\phonon_backend\\*d4.dll"
    file.puts "\tdel /F /Q bin\\plugins\\sqldrivers\\*d.dll"
    file.puts "\tdel /F /Q bin\\plugins\\sqldrivers\\*d4.dll"
  else
    file.puts "all: clean build"
    file.puts ""
    file.puts "makedirs:"
    file.puts "\t-mkdir ext/build"
    file.puts "\t-mkdir bin/1.8"
    file.puts "\t-mkdir bin/1.9"
    file.puts "\t-mkdir bin/plugins"
    file.puts "\t-mkdir bin/plugins/accessible"
    file.puts "\t-mkdir bin/plugins/codecs"
    file.puts "\t-mkdir bin/plugins/designer"
    file.puts "\t-mkdir bin/plugins/graphicssystems"
    file.puts "\t-mkdir bin/plugins/iconengines"
    file.puts "\t-mkdir bin/plugins/imageformats"
    file.puts "\t-mkdir bin/plugins/phonon_backend"
    file.puts "\t-mkdir bin/plugins/sqldrivers"
    file.puts "\t-mkdir lib/1.8"
    file.puts "\t-mkdir lib/1.9"    
    file.puts ""
    file.puts "clean: makedirs"
    file.puts "\t-cd ext/build; rm -rf CMakeFiles"
    file.puts "\t-cd ext/build; rm -rf generator"
    file.puts "\t-cd ext/build; rm -rf smoke"
    file.puts "\t-cd ext/build; rm -rf ruby"
    file.puts "\t-cd ext/build; rm *"
    file.puts ""
    file.puts "distclean: clean"
    file.puts "\t-cd bin && rm *.dll"
    file.puts "\t-cd bin && rm *.so"
    file.puts "\t-cd bin && rm *.exe"
    file.puts "\t-cd bin/plugins && rm *"
    file.puts "\t-cd bin/plugins/accessible && rm *"
    file.puts "\t-cd bin/plugins/codecs && rm *"
    file.puts "\t-cd bin/plugins/designer && rm *"
    file.puts "\t-cd bin/plugins/graphicssystems && rm *"
    file.puts "\t-cd bin/plugins/iconengines && rm *"
    file.puts "\t-cd bin/plugins/imageformats && rm *"
    file.puts "\t-cd bin/plugins/phonon_backend && rm *"
    file.puts "\t-cd bin/plugins/sqldrivers && rm *"
    file.puts "\t-cd bin/1.8 && rm *"
    file.puts "\t-cd bin/1.9 && rm *"
    file.puts "\t-cd lib/1.8 && rm *"
    file.puts "\t-cd lib/1.9 && rm *"    
    file.puts "\t-rm Makefile"
    file.puts "\t-rm qtbindings-*.gem"
    file.puts ""
    file.puts "build: makedirs"
    file.puts "\t-cd ext/build; \\"
    file.puts "cmake \\"
    file.puts "-G \"Unix Makefiles\" \\"
    if ARGV[0] == '-d'
      file.puts "-DCMAKE_BUILD_TYPE=Debug \\"
    end   
    file.puts "-Wno-dev \\"
    file.puts "-DENABLE_SMOKE=on \\"
    file.puts "-DENABLE_QTCORE_SMOKE=on \\"
    file.puts "-DENABLE_QTNETWORK_SMOKE=on \\"
    file.puts "-DENABLE_QTDBUS_SMOKE=on \\"
    file.puts "-DENABLE_QTGUI_SMOKE=on \\"
    file.puts "-DENABLE_QTSVG_SMOKE=on \\"
    file.puts "-DENABLE_QTSQL_SMOKE=on \\"
    file.puts "-DENABLE_QTXML_SMOKE=on \\"
    file.puts "-DENABLE_QTXMLPATTERNS_SMOKE=on \\"
    file.puts "-DENABLE_QTOPENGL_SMOKE=on \\"
    file.puts "-DENABLE_QTWEBKIT_SMOKE=on \\"
    file.puts "-DENABLE_QTSCRIPT_SMOKE=on \\"
    file.puts "-DENABLE_QTUITOOLS_SMOKE=on \\"
    file.puts "-DENABLE_QTTEST_SMOKE=on \\"
    file.puts "-DENABLE_QTMULTIMEDIA_SMOKE=on \\"
    file.puts "-DENABLE_QTRUBY=on \\"
    file.puts "-DENABLE_QTWEBKIT_RUBY=on \\"
    file.puts "-DENABLE_QTUITOOLS_RUBY=on \\"
    file.puts "-DENABLE_QTSCRIPT=on \\"
    file.puts "-DENABLE_QTTEST=on \\"
    file.puts ".."
    file.puts "\tcd ext/build; make"
    file.puts ""    
    file.puts "install: makedirs"
    if macosx
      file.puts "\t-cp ext/build/smoke/smokeapi/smokeapi bin/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/deptool/smokedeptool bin/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtruby/src/qtruby4.so lib/#{ruby_version}/qtruby4.bundle"
      file.puts "\t-cp ext/build/ruby/qtscript/qtscript.* lib/#{ruby_version}/qtscript.bundle"
      file.puts "\t-cp ext/build/ruby/qttest/qttest.* lib/#{ruby_version}/qttest.bundle"
      file.puts "\t-cp ext/build/ruby/qtuitools/qtuitools.* lib/#{ruby_version}/qtuitools.bundle"
      file.puts "\t-cp ext/build/ruby/qtwebkit/qtwebkit.* lib/#{ruby_version}/qtwebkit.bundle"
      file.puts "\t-cp ext/build/ruby/qtruby/tools/rbrcc/rbrcc bin/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtruby/tools/rbuic/rbuic4 bin/#{ruby_version}"
    else
      file.puts "\t-cp ext/build/smoke/deptool/smokedeptool bin/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtcore/libsmokeqtcore.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtdbus/libsmokeqtdbus.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtgui/libsmokeqtgui.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qthelp/libsmokeqthelp.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtmultimedia/libsmokeqtmultimedia.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtnetwork/libsmokeqtnetwork.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtopengl/libsmokeqtopengl.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtscript/libsmokeqtscript.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtsql/libsmokeqtsql.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtsvg/libsmokeqtsvg.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qttest/libsmokeqttest.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtuitools/libsmokeqtuitools.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtwebkit/libsmokeqtwebkit.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtxml/libsmokeqtxml.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/qtxmlpatterns/libsmokeqtxmlpatterns.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/smokeapi/smokeapi bin/#{ruby_version}"
      file.puts "\t-cp ext/build/smoke/smokebase/libsmokebase.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtruby/src/libqtruby4shared.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtruby/src/qtruby4.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtscript/qtscript.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qttest/qttest.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtuitools/qtuitools.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtwebkit/qtwebkit.* lib/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtruby/tools/rbrcc/rbrcc bin/#{ruby_version}"
      file.puts "\t-cp ext/build/ruby/qtruby/tools/rbuic/rbuic4 bin/#{ruby_version}"
    end
  end
end
